@@ -1,7 +1,7 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
 /* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
-   Copyright (c) 2006 - 2012 Thomas Schmitt <scdbackup@gmx.net>
+   Copyright (c) 2006 - 2013 Thomas Schmitt <scdbackup@gmx.net>
    Provided under GPL version 2 or later.
 */
 
@@ -31,6 +31,7 @@
 #include "libburn.h"
 #include "drive.h"
 #include "transport.h"
+#include "util.h"
 
 /* ts A60825 : The storage location for back_hacks.h variables. */
 #define BURN_BACK_HACKS_INIT 1
@@ -43,6 +44,9 @@ struct libdax_msgs *libdax_messenger= NULL;
 
 int burn_running = 0;
 
+double lib_start_time;
+
+
 /* ts A60813 : GNU/Linux: whether to use O_EXCL on open() of device files
    ts B00212 : FreeBSD:   whether to use flock(LOCK_EX) after open()
 */
@@ -136,6 +140,8 @@ int burn_initialize(void)
 
 	if (burn_running)
 		return 1;
+
+	lib_start_time = burn_get_time(0);
 	burn_support_untested_profiles = 0;
 	ret = burn_msgs_initialize();
 	if (ret <= 0)
@@ -347,6 +353,17 @@ int burn_sev_to_text(int severity_number, char **severity_name, int flag)
 	return ret;
 }
 
+
+/* ts B21214 API */
+char *burn_list_sev_texts(int flag)
+{
+ char *sev_list;
+
+ libdax_msgs__sev_to_text(0, &sev_list, 1);
+ return sev_list;
+}
+
+
 /* ts B00224 */
 char *burn_util_thread_id(pid_t pid, pthread_t tid, char text[80])
 {
@@ -532,7 +549,7 @@ void burn_set_signal_handling(void *handle, burn_abort_handler_t handler,
 	if(burn_builtin_signal_action == 0)
 		burn_builtin_signal_action = 1;
 	Cleanup_set_handlers(handle, (Cleanup_app_handler_T) handler,
-				 (mode & 15) | 4);
+				 (mode & 15) | 4 | (mode & 256));
 	burn_global_signal_handle = handle;
 	burn_global_signal_handler = handler;
 }
