@@ -1,6 +1,6 @@
 
 /* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
-   Copyright (c) 2006 - 2012 Thomas Schmitt <scdbackup@gmx.net>
+   Copyright (c) 2006 - 2017 Thomas Schmitt <scdbackup@gmx.net>
    Provided under GPL version 2 or later.
 */
 
@@ -44,7 +44,8 @@ struct burn_write_opts *burn_write_opts_new(struct burn_drive *drive)
 	opts->toc_entry = NULL;
 	opts->toc_entries = 0;
 	opts->simulate = 0;
-	opts->underrun_proof = drive->mdata->underrun_proof;
+	opts->underrun_proof = drive->mdata->p2a_valid > 0 &&
+	                       drive->mdata->underrun_proof;
 	opts->perform_opc = 1;
 	opts->obs = -1;
 
@@ -88,12 +89,22 @@ int burn_write_opts_clone(struct burn_write_opts *from,
 		return 1;
 	*to = calloc(1, sizeof(struct burn_write_opts));
 	if (*to == NULL) {
+out_of_mem:;
 		libdax_msgs_submit(libdax_messenger, -1, 0x00000003,
 				LIBDAX_MSGS_SEV_FATAL, LIBDAX_MSGS_PRIO_HIGH,
 				"Out of virtual memory", 0, 0);
 		return -1;
 	}
 	memcpy(*to, from, sizeof(struct burn_write_opts));
+	(*to)->text_packs = NULL;
+	(*to)->num_text_packs = 0;
+	if (from->text_packs != NULL && from->num_text_packs > 0) {
+		(*to)->text_packs = calloc(1, from->num_text_packs * 18);
+		if ((*to)->text_packs == NULL)
+			goto out_of_mem;
+		memcpy((*to)->text_packs, from->text_packs,
+		       from->num_text_packs * 18);
+	}
 	(*to)->refcount= 1;
 	return 1;
 }
@@ -172,17 +183,6 @@ void burn_write_opts_set_format(struct burn_write_opts *opts, int format)
 
 int burn_write_opts_set_simulate(struct burn_write_opts *opts, int sim)
 {
-/* <<< ts A70529 :
-       One cannot predict the ability to simulate from page 05h
-       information alone. This check is now done later in 
-       function  burn_write_opts_auto_write_type().
-
-	if (opts->drive->mdata->simulate) {
-		opts->simulate = sim;
-		return 1;
-	}
-	return 0;
-*/
 	opts->simulate = !!sim;
 	return 1;
 }
@@ -190,9 +190,8 @@ int burn_write_opts_set_simulate(struct burn_write_opts *opts, int sim)
 int burn_write_opts_set_underrun_proof(struct burn_write_opts *opts,
 				       int underrun_proof)
 {
-	if (opts->drive->mdata->valid <= 0)
-		return 0;
-	if (opts->drive->mdata->underrun_proof) {
+	if (opts->drive->mdata->p2a_valid <= 0 ||
+	    opts->drive->mdata->underrun_proof) {
 		opts->underrun_proof = underrun_proof;
 		return 1;
 	}
@@ -289,10 +288,12 @@ int burn_write_opts_set_leadin_text(struct burn_write_opts *opts,
 	if (num_packs > 0) {
 		memcpy(pack_buffer, text_packs, num_packs * 18);
 		opts->text_packs = pack_buffer;
+		pack_buffer = NULL;
 	}
 	opts->num_text_packs = num_packs;
 	ret = 1;
 ex:;
+	BURN_FREE_MEM(pack_buffer);
 	return ret;
 }
 
@@ -322,6 +323,11 @@ enum burn_write_types burn_write_opts_auto_write_type(
 
 	reasons[0] = 0;
 
+	if (burn_drive_get_bd_r_pow(d)) {
+		strcat(reasons,
+		       "MEDIA: unsuitable BD-R Pseudo Overwrite formatting, ");
+		return BURN_WRITE_NONE;
+	}
 	if (d->status != BURN_DISC_BLANK &&
 	    d->status != BURN_DISC_APPENDABLE){
 		if (d->status == BURN_DISC_FULL)
@@ -533,9 +539,11 @@ void burn_write_opts_set_obs_pad(struct burn_write_opts *opts, int pad)
 void burn_write_opts_set_stdio_fsync(struct burn_write_opts *opts, int rythm)
 {
 	if (rythm == -1)
-		opts->stdio_fsync_size = 0;
+		opts->stdio_fsync_size = -1; /* never */
 	else if (rythm == 0)
 		opts->stdio_fsync_size = Libburn_stdio_fsync_limiT;
+	else if (rythm == 1)
+		opts->stdio_fsync_size = 0; /* only at end of writing */
 	else if (rythm >= 32)
 		opts->stdio_fsync_size = rythm;
 }
