@@ -1,5 +1,14 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
+/* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
+   Copyright (c) 2006 - 2016 Thomas Schmitt <scdbackup@gmx.net>
+   Provided under GPL version 2 or later.
+*/
+
+#ifdef HAVE_CONFIG_H
+#include "../config.h"
+#endif
+
 #include <stdio.h>
 
 /* ts A61010 */
@@ -15,18 +24,26 @@
 #include "sector.h"
 #include "crc.h"
 #include "debug.h"
-#include "lec.h"
 #include "toc.h"
 #include "write.h"
 
+#include "libdax_msgs.h"
+extern struct libdax_msgs *libdax_messenger;
+
+#include "ecma130ab.h"
+
 
 #ifdef Libburn_log_in_and_out_streaM
-/* <<< ts A61031 */
+/* ts A61031 */
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #endif /* Libburn_log_in_and_out_streaM */
 
+/* ts B41126 : O_BINARY is needed for Cygwin but undefined elsewhere */
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
 
 /*static unsigned char isrc[] = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";*/
 
@@ -92,11 +109,12 @@ static void get_bytes(struct burn_track *track, int count, unsigned char *data)
 	int valid, shortage, curr, i, tr;
 
 #ifdef Libburn_log_in_and_out_streaM 
-        /* <<< ts A61031 */
+        /* ts A61031 */
         static int tee_fd= -1;
         if(tee_fd==-1)
                 tee_fd= open("/tmp/libburn_sg_readin",
-                                O_WRONLY|O_CREAT|O_TRUNC,S_IRUSR|S_IWUSR);
+				O_WRONLY | O_CREAT | O_TRUNC | O_BINARY,
+				S_IRUSR | S_IWUSR);
 #endif /* Libburn_log_in_and_out_streaM */
 
 
@@ -138,7 +156,7 @@ static void get_bytes(struct burn_track *track, int count, unsigned char *data)
 	track->sourcecount += valid;
 
 #ifdef Libburn_log_in_and_out_streaM
-	/* <<< ts A61031 */
+	/* ts A61031 */
         if(tee_fd!=-1 && valid>0) {
                 write(tee_fd, data + curr, valid);
         }
@@ -166,10 +184,27 @@ static void get_bytes(struct burn_track *track, int count, unsigned char *data)
 	if (!shortage)
 		goto ex;
 
-	/* ts A61031 */
+	/* ts A61031 - B10103 */
 	if (shortage >= count)
 		track->track_data_done = 1;
-	if (track->open_ended)
+	if (track->end_on_premature_eoi && shortage >= count &&
+	    !track->open_ended) {
+		char msg[80];
+		off_t missing, inp_block_size, track_blocks;
+
+		inp_block_size = burn_sector_length(track->mode);
+		track_blocks = burn_track_get_sectors_2(track, 1);
+		missing = track_blocks * inp_block_size - track->sourcecount;
+		sprintf(msg,
+		      "Premature end of input encountered. Missing: %.f bytes",
+		      (double) missing);
+		libdax_msgs_submit(libdax_messenger, -1, 0x00020180,
+				LIBDAX_MSGS_SEV_FAILURE, LIBDAX_MSGS_PRIO_HIGH,
+				msg, 0,0);
+		/* Memorize that premature end of input happened */
+		track->end_on_premature_eoi = 2;
+	}
+	if (track->open_ended || track->end_on_premature_eoi)
 		goto ex;
 
 /* If we're still short, and there's a "next" pointer, we pull from that.
@@ -198,6 +233,43 @@ ex:;
 	}
 }
 
+
+/* ts B20113 : outsourced from get_sector() */
+int sector_write_buffer(struct burn_drive *d, 
+			struct burn_track *track, int flag)
+{
+	int err, i;
+	struct buffer *out;
+
+	out = d->buffer;
+	if (out->sectors <= 0)
+		return 2;
+	err = d->write(d, d->nwa, out);
+	if (err == BE_CANCELLED)
+		return 0;
+
+	/* ts A61101 */
+	if(track != NULL) {
+		track->writecount += out->bytes;
+		track->written_sectors += out->sectors;
+
+		/* Determine current index */
+		for (i = d->progress.index; i + 1 < track->indices; i++) {
+			if (track->index[i + 1] > d->nwa + out->sectors)
+		break;
+			d->progress.index = i + 1;
+		}
+	}
+	/* ts A61119 */
+	d->progress.buffered_bytes += out->bytes;
+
+	d->nwa += out->sectors;
+	out->bytes = 0;
+	out->sectors = 0;
+	return 1;
+}
+
+
 /* ts A61009 : seems to hand out sector start pointer in opts->drive->buffer
 		and to count hand outs as well as reserved bytes */
 /* ts A61101 : added parameter track for counting written bytes */
@@ -206,8 +278,7 @@ static unsigned char *get_sector(struct burn_write_opts *opts,
 {
 	struct burn_drive *d = opts->drive;
 	struct buffer *out = d->buffer;
-	int outmode;
-	int seclen;
+	int outmode, seclen, write_ret;
 	unsigned char *ret;
 
 	outmode = get_outmode(opts);
@@ -223,26 +294,13 @@ static unsigned char *get_sector(struct burn_write_opts *opts,
 	seclen += burn_subcode_length(outmode);
 
 	/* ts A61219 : opts->obs is eventually a 32k trigger for DVD */
+	/* (there is enough buffer size reserve for track->cdxa_conversion) */
 	if (out->bytes + seclen > BUFFER_SIZE ||
 	    (opts->obs > 0 && out->bytes + seclen > opts->obs)) {
-		int err;
-		err = d->write(d, d->nwa, out);
-		if (err == BE_CANCELLED)
+		write_ret = sector_write_buffer(d, track, 0);
+		if (write_ret <= 0)
 			return NULL;
-
-		/* ts A61101 */
-		if(track != NULL) {
-			track->writecount += out->bytes;
-			track->written_sectors += out->sectors;
-		}
-		/* ts A61119 */
-		d->progress.buffered_bytes += out->bytes;
-
-		d->nwa += out->sectors;
-		out->bytes = 0;
-		out->sectors = 0;
 	}
-
 	ret = out->data + out->bytes;
 	out->bytes += seclen;
 	out->sectors++;
@@ -295,11 +353,21 @@ static int convert_data(struct burn_write_opts *o, struct burn_track *track,
 
 	/* ts A61010 */
 	/* a ssert(outlen >= inlen); */
-	if (outlen < inlen)
+	if (outlen < inlen || outlen < 0 || inlen < 0)
 		return 0;
 
 	if ((outmode & BURN_MODE_BITS) == (inmode & BURN_MODE_BITS)) {
+		/* see MMC-5 4.2.3.8.5.3 Block Format for Mode 2 form 1 Data
+		            Table 24  Mode 2 Formed Sector Sub-header Format */
+		if (track != NULL)
+			if (track->cdxa_conversion == 1)
+				inlen += 8;
+
 		get_bytes(track, inlen, data);
+
+		if (track != NULL)
+			if (track->cdxa_conversion == 1)
+				memmove(data, data + 8, inlen - 8);
 		return 1;
 	}
 
@@ -392,7 +460,13 @@ a2 - lout ctrl
 	q[7] = dec_to_bcd(d->toc_entry[track].pmin);
 	q[8] = dec_to_bcd(d->toc_entry[track].psec);
 	q[9] = dec_to_bcd(d->toc_entry[track].pframe);
+
+#ifdef Libburn_no_crc_C
+	crc = 0; /* dummy */
+#else
 	crc = crc_ccitt(q, 10);
+#endif
+
 	q[10] = crc >> 8;
 	q[11] = crc & 0xFF;
 	d->toc_temp++;
@@ -413,7 +487,8 @@ int sector_toc(struct burn_write_opts *o, int mode)
 		return 0;
 	subcode_toc(d, mode, subs);
 	convert_subs(o, mode, subs, data);
-	sector_headers(o, data, mode, 1);
+	if (sector_headers(o, data, mode, 1) <= 0)
+		return 0;
 	sector_common(++)
 	return 1;
 }
@@ -433,7 +508,8 @@ int sector_pregap(struct burn_write_opts *o,
 		return 0;
 	subcode_user(o, subs, tno, control, 0, NULL, 1);
 	convert_subs(o, mode, subs, data);
-	sector_headers(o, data, mode, 0);
+	if (sector_headers(o, data, mode, 0) <= 0)
+		return 0;
 	sector_common(--)
 	return 1;
 }
@@ -450,11 +526,12 @@ int sector_postgap(struct burn_write_opts *o,
 		return 0;
 	/* ts A61010 */
 	if (convert_data(o, NULL, mode, data) <= 0)
-		return 0;;
+		return 0;
 /* use last index in track */
 	subcode_user(o, subs, tno, control, 1, NULL, 1);
 	convert_subs(o, mode, subs, data);
-	sector_headers(o, data, mode, 0);
+	if (sector_headers(o, data, mode, 0) <= 0)
+		return 0;
 	sector_common(++)
 	return 1;
 }
@@ -486,7 +563,13 @@ static void subcode_lout(struct burn_write_opts *o, unsigned char control,
 	q[7] = dec_to_bcd(min);
 	q[8] = dec_to_bcd(sec);
 	q[9] = dec_to_bcd(frame);
+
+#ifdef Libburn_no_crc_C
+	crc = 0; /* dummy */
+#else
 	crc = crc_ccitt(q, 10);
+#endif
+
 	q[10] = crc >> 8;
 	q[11] = crc & 0xFF;
 }
@@ -606,7 +689,13 @@ void subcode_user(struct burn_write_opts *o, unsigned char *subcodes,
 	}
 	q[0] = (control << 4) + qmode;
 
+
+#ifdef Libburn_no_crc_C
+	crc = 0; /* dummy */
+#else
 	crc = crc_ccitt(q, 10);
+#endif
+
 	q[10] = crc >> 8;
 	q[11] = crc & 0xff;
 }
@@ -625,7 +714,8 @@ int sector_lout(struct burn_write_opts *o, unsigned char control, int mode)
 		return 0;
 	subcode_lout(o, control, subs);
 	convert_subs(o, mode, subs, data);
-	sector_headers(o, data, mode, 0);
+	if (sector_headers(o, data, mode, 0) <= 0)
+		return 0;
 	sector_common(++)
 	return 1;
 }
@@ -637,14 +727,14 @@ int sector_data(struct burn_write_opts *o, struct burn_track *t, int psub)
 	unsigned char *data;
 
 	data = get_sector(o, t, t->mode);
-	if (!data)
+	if (data == NULL)
 		return 0;
 	/* ts A61010 */
 	if (convert_data(o, t, t->mode, data) <= 0)
 		return 0;
 
 	/* ts A61031 */
-	if (t->open_ended && t->track_data_done) {
+	if ((t->open_ended || t->end_on_premature_eoi) && t->track_data_done) {
 		unget_sector(o, t->mode);
 		return 2;
 	}
@@ -660,7 +750,8 @@ int sector_data(struct burn_write_opts *o, struct burn_track *t, int psub)
 			     t->entry->control, 1, &t->isrc, psub);
 	convert_subs(o, t->mode, subs, data);
 
-	sector_headers(o, data, t->mode, 0);
+	if (sector_headers(o, data, t->mode, 0) <= 0)
+		return 0;
 	sector_common(++)
 	return 1;
 }
@@ -711,42 +802,27 @@ int sector_headers_is_ok(struct burn_write_opts *o, int mode)
 	return 0;
 }
 
-void sector_headers(struct burn_write_opts *o, unsigned char *out,
+/* ts A90830 : changed return type to int
+   @return 0= failure
+           1= success
+*/
+int sector_headers(struct burn_write_opts *o, unsigned char *out,
 		    int mode, int leadin)
 {
+
+#ifdef Libburn_ecma130ab_includeD
+
 	struct burn_drive *d = o->drive;
 	unsigned int crc;
 	int min, sec, frame;
 	int modebyte = -1;
-
-	/* ts A61009 */
-#if 1
 	int ret;
 
 	ret = sector_headers_is_ok(o, mode);
 	if (ret != 2)
-		return;
+		return !!ret;
 	modebyte = 1;
 
-#else
-
-	if (mode & BURN_AUDIO)	/* no headers for "audio" */
-		return;
-	if (o->write_type == BURN_WRITE_SAO)
-		return;
-
-	/* ts A61031 */
-	if (o->write_type == BURN_WRITE_TAO)
-		return;
-
-	if (mode & BURN_MODE1)
-		modebyte = 1;
-
-#endif
-
-	/* ts A61009 : now ensured by burn_disc_write_is_ok() */
-	/* a ssert(modebyte == 1); */
-
 	out[0] = 0;
 	memset(out + 1, 0xFF, 10);	/* sync */
 	out[11] = 0;
@@ -765,7 +841,13 @@ void sector_headers(struct burn_write_opts *o, unsigned char *out,
 		out[15] = modebyte;
 	}
 	if (mode & BURN_MODE1) {
+
+#ifdef Libburn_no_crc_C
+		crc = 0; /* dummy */
+#else
 		crc = crc_32(out, 2064);
+#endif
+
 		out[2064] = crc & 0xFF;
 		crc >>= 8;
 		out[2065] = crc & 0xFF;
@@ -776,10 +858,33 @@ void sector_headers(struct burn_write_opts *o, unsigned char *out,
 	}
 	if (mode & BURN_MODE1) {
 		memset(out + 2068, 0, 8);
-		parity_p(out);
-		parity_q(out);
+		burn_rspc_parity_p(out);
+		burn_rspc_parity_q(out);
 	}
-	scramble(out);
+	burn_ecma130_scramble(out);
+	return 1;
+
+#else /* Libburn_ecma130ab_includeD */
+
+	int ret;
+
+	ret = sector_headers_is_ok(o, mode);
+	if (ret != 2)
+		return (!! ret);
+
+	/* ts A90830 : lec.c is copied from cdrdao.
+	   I have no idea yet how lec.c implements the Reed-Solomon encoding
+	   which is described in ECMA-130 for CD-ROM.
+	   So this got removed for now.
+	*/
+	libdax_msgs_submit(libdax_messenger, o->drive->global_index,
+				0x0002010a,
+				LIBDAX_MSGS_SEV_FATAL, LIBDAX_MSGS_PRIO_HIGH,
+				"Raw CD write modes are not supported", 0, 0);
+	return 0;
+
+#endif /* ! Libburn_ecma130ab_includeD */
+
 }
 
 #if 0
@@ -807,7 +912,7 @@ void process_q(struct burn_drive *d, unsigned char *q)
 		}
 		break;
 	case 2:
-		/* XXX dont ignore these */
+		/* XXX do not ignore these */
 		break;
 	case 3:
 /*              burn_print(12, "ISRC data in mode 3 q\n");*/
@@ -826,7 +931,7 @@ void process_q(struct burn_drive *d, unsigned char *q)
 
 /* this needs more info.  subs in the data? control/adr? */
 
-/* ts A61119 : One should not use inofficial compiler extensions.
+/* ts A61119 : One should not use unofficial compiler extensions.
    >>> Some day this function needs to be implemented. At least for now
        the result does not match the "mode" of cdrecord -toc.
  */
@@ -835,12 +940,14 @@ void process_q(struct burn_drive *d, unsigned char *q)
 */
 int sector_identify(unsigned char *data)
 {
-	scramble(data);
+
 /*
+	burn_ecma130_scramble(data);
 check mode byte for 1 or 2
 test parity to see if it's a valid sector
 if invalid, return BURN_MODE_AUDIO;
 else return mode byte  (what about mode 2 formless?  heh)
 */
+
 	return BURN_MODE1;
 }
