@@ -1,7 +1,7 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
 /*
-   Copyright (c) 2009 - 2014 Thomas Schmitt <scdbackup@gmx.net>
+   Copyright (c) 2009 - 2016 Thomas Schmitt <scdbackup@gmx.net>
    Provided under GPL version 2 or later.
 */
 
@@ -45,7 +45,7 @@ sg_initialize()         performs global initialization of the SCSI transport
                         software components.
 
 sg_shutdown()           performs global finalizations and releases golbally
-                        aquired resources.
+                        acquired resources.
 
 sg_give_next_adr()      iterates over the set of potentially useful drive 
                         address strings.
@@ -54,7 +54,7 @@ scsi_enumerate_drives() brings all available, not-whitelist-banned, and
                         accessible drives into libburn's list of drives.
 
 sg_dispose_drive()      finalizes adapter specifics of struct burn_drive
-                        on destruction. Releases resources which were aquired
+                        on destruction. Releases resources which were acquired
                         underneath scsi_enumerate_drives().
  
 sg_drive_is_open()      tells wether libburn has the given drive in use.
@@ -78,7 +78,7 @@ burn_os_is_2k_seekrw()  tells whether the given path leads to a file object
 
 burn_os_stdio_capacity()  estimates the emulated media space of stdio-drives.
 
-burn_os_open_track_src()  opens a disk file in a way that allows best
+burn_os_open_track_src()  opens a disk file in a way that offers best
                         throughput with file reading and/or SCSI write command
                         transmission.
 
@@ -395,7 +395,7 @@ int sg_initialize(char msg[1024], int flag)
 
 
 /** Performs global finalization of the SCSI transport adapter and eventually
-    needed operating system facilities. Releases globally aquired resources.
+    needed operating system facilities. Releases globally acquired resources.
     @param flag  unused yet, submit 0
     @return      1 = success, <=0 = failure
 */ 
@@ -529,7 +529,7 @@ ex:;
 
 /** Tells whether libburn has the given drive in use or exclusively reserved.
     If it is "open" then libburn will eventually call sg_release() on it when
-    it is time to give up usage resp. reservation.
+    it is time to give up usage and reservation.
 */
 /** Published as burn_drive.drive_is_open() */
 int sg_drive_is_open(struct burn_drive * d)
@@ -665,6 +665,9 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 		e_direction = SCSI_MMC_DATA_NONE;
 	}
 		
+	/* ts B90523 : Record effective transfer length request for debugging*/
+	c->dxfer_len = dxfer_len;
+
 	/* retry-loop */
 	start_time = time(NULL);
 	if (c->timeout > 0)
@@ -728,6 +731,8 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 					     start_time, timeout_ms, i, 0);
 		if (d->cancel)
 			done = 1;
+		if (!done)
+			spc_register_retry(c);
 
 	} /* end of retry-loop */
 
@@ -840,7 +845,7 @@ static int freebsd_is_2k_seekrw(char *path, int flag)
 
 
 /* Return 1 if the given path leads to a regular file or a device that can be
-   seeked, read, and possibly written with 2 kB granularity. 
+   fseeked, read, and possibly written with 2 kB granularity. 
 */
 int burn_os_is_2k_seekrw(char *path, int flag)
 {
