@@ -1,5 +1,15 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
+/* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
+   Copyright (c) 2011 - 2011 Thomas Schmitt <scdbackup@gmx.net>
+   Provided under GPL version 2 or later.
+*/
+
+
+#ifdef HAVE_CONFIG_H
+#include "../config.h"
+#endif
+
 /* ts A61008 */
 /* #include <a ssert.h> */
 
@@ -11,6 +21,7 @@
 #include "libburn.h"
 #include "sector.h"
 #include "options.h"
+#include "init.h"
 
 #if 0
 static void write_clonecd2(volatile struct toc *toc, int f);
@@ -94,17 +105,14 @@ static void write_clonecd2(volatile struct toc *toc, int f)
 
 void toc_find_modes(struct burn_drive *d)
 {
-	struct burn_read_opts o;
-	int lba;
 	int i, j;
-	struct buffer mem;
+	struct buffer *mem = NULL;
 	struct burn_toc_entry *e;
 
-	/* ts A61008 : to be prevented on the higher levels */
-	/* a ssert(d->busy); */
+/* ts A70519 : the code which needs this does not work with GNU/Linux 2.4 USB
+	int lba;
+	struct burn_read_opts o;
 
-	mem.bytes = 0;
-	mem.sectors = 1;
 	o.raw = 1;
 	o.c2errors = 0;
 	o.subcodes_audio = 1;
@@ -113,29 +121,42 @@ void toc_find_modes(struct burn_drive *d)
 	o.report_recovered_errors = 0;
 	o.transfer_damaged_blocks = 1;
 	o.hardware_error_retries = 1;
+*/
+
+	BURN_ALLOC_MEM_VOID(mem, struct buffer, 1);
+
+	mem->bytes = 0;
+	mem->sectors = 1;
 
 	for (i = 0; i < d->disc->sessions; i++)
 		for (j = 0; j < d->disc->session[i]->tracks; j++) {
 			struct burn_track *t = d->disc->session[i]->track[j];
 
 			e = t->entry;
-			if (!e)
-				lba = 0;
-			else
-				lba = burn_msf_to_lba(e->pmin, e->psec,
-						      e->pframe);
 /* XXX | in the subcodes if appropriate! */
 			if (e && !(e->control & 4)) {
 				t->mode = BURN_AUDIO;
 			} else {
 
 				t->mode = BURN_MODE1;
-/* ts A70519 : this does not work with Linux 2.4 USB because one cannot
+/* ts A70519 : this does not work with GNU/Linux 2.4 USB because one cannot
                predict the exact dxfer_size without knowing the sector type.
-				mem.sectors = 1;
-				d->read_sectors(d, lba, mem.sectors, &o, &mem);
-				t->mode = sector_identify(mem.data);
+				if (!e)
+					lba = 0;
+				else
+					lba = burn_msf_to_lba(e->pmin, e->psec,
+							      e->pframe);
+				mem->sectors = 1;
+
+				ts B21119 : Would now be d->read_cd() with
+					    with sectype = 0 , mainch = 0xf8
+				d->read_sectors(d, lba, mem.sectors, &o, mem);
+
+				t->mode = sector_identify(mem->data);
 */
 			}
 		}
+
+ex:
+	BURN_FREE_MEM(mem);
 }
