@@ -440,7 +440,7 @@ static int add_isrc_cue(struct cue_sheet *sheet, unsigned char ctladr, int tno,
 {
 	unsigned char *unit;
 	int i, ret;
-	char text[8];
+	char text[8 + 21]; /* should suffice for 64 bit oversize */
 
 	ret = new_cue(sheet, 2, 0);
 	if (ret <= 0)
@@ -454,6 +454,9 @@ static int add_isrc_cue(struct cue_sheet *sheet, unsigned char ctladr, int tno,
 	unit[5] = isrc->owner[1];
 	unit[6] = isrc->owner[2];
 	sprintf(text, "%-2.2u%-5.5u", (unsigned int) isrc->year, isrc->serial);
+	sprintf(text, "%-2.2u", (unsigned int) isrc->year);
+	sprintf(text + 2, "%-5.5u", isrc->serial);
+	text[7] = 0;
 	unit[7] = text[0];
 	for (i = 1; i < 7; i++)
 		unit[9 + i] = text[i];
@@ -1376,6 +1379,7 @@ int burn_disc_init_write_status(struct burn_write_opts *o,
 	ret = burn_write_opts_clone(o, &(d->write_opts), 0);
 	if (ret <= 0)
 		return ret;
+	d->write_retry_count = 0;
 
 	d->busy = BURN_DRIVE_WRITING;
 
@@ -1702,7 +1706,7 @@ int burn_disc_close_track_dvd_minus_r(struct burn_write_opts *o, int tnum)
 int burn_disc_finalize_dvd_plus_r(struct burn_write_opts *o)
 {
 	struct burn_drive *d = o->drive;
-	char msg[80];
+	char msg[40 + 80]; /* filltext + profile */
 
 	sprintf(msg, "Finalizing %s ...",
 		d->current_profile_text);
@@ -2922,7 +2926,7 @@ void burn_disc_write_sync(struct burn_write_opts *o, struct burn_disc *disc)
 	struct buffer *buffer_mem = o->drive->buffer;
 	struct burn_session *s;
 	struct burn_track *lt, *t;
-	int first = 1, i, ret, lba, nwa = 0, multi_mem;
+	int first = 1, i, ret, lba, nwa = 0, multi_mem, stream_recording_start;
 	off_t default_size;
 	char msg[80];
 
@@ -2934,12 +2938,13 @@ void burn_disc_write_sync(struct burn_write_opts *o, struct burn_disc *disc)
 	/* ts A61224 */
 	burn_disc_init_write_status(o, disc); /* must be done very early */
 
-	/* ts A80412 , A90227 */
-	d->do_stream_recording = !!o->do_stream_recording;
+	/* ts A80412 , A90227 , B90411 */
 	if (o->do_stream_recording >= 16)
-		d->stream_recording_start = o->do_stream_recording;
+		stream_recording_start = o->do_stream_recording;
 	else
-		d->stream_recording_start = 0;
+		stream_recording_start = 0;
+	burn_drive_set_stream_recording(d, !!o->do_stream_recording,
+					stream_recording_start, 0);
 
 	/* ts A91122 : Get buffer suitable for sources made by
 	               burn_os_open_track_src() */
@@ -3190,6 +3195,14 @@ ex:;
 		burn_write_opts_free(d->write_opts);
 		d->write_opts = NULL;
 	}
+	if (d->write_retry_count > 0) {
+		sprintf(msg, "WRITE command repetition happened %u times",
+			d->write_retry_count);
+		libdax_msgs_submit(libdax_messenger, d->global_index,
+			0x000201ad,
+			LIBDAX_MSGS_SEV_NOTE, LIBDAX_MSGS_PRIO_HIGH,
+			msg, 0, 0);
+	}
 	return;
 }
 
