@@ -1,6 +1,6 @@
 
 /*  test/telltoc.c , API illustration of obtaining media status info */
-/*  Copyright (C) 2006 - 2011 Thomas Schmitt <scdbackup@gmx.net> 
+/*  Copyright (C) 2006 - 2016 Thomas Schmitt <scdbackup@gmx.net> 
     Provided under GPL */
 
 /**                               Overview 
@@ -14,15 +14,16 @@
   to serve the libburn team as reference application. telltoc.c does indeed
   define the standard way how above gestures can be implemented and stay upward
   compatible for a good while.
+  The burn aspects of libburn are demonstrated by program test/libburner.c .
   
   Before you can do anything, you have to initialize libburn by
      burn_initialize()
-  as it is done in main() at the end of this file. Then you aquire a
+  as it is done in main() at the end of this file. Then you acquire a
   drive in an appropriate way conforming to the API. The two main
   approaches are shown here in application functions:
      telltoc_aquire_by_adr()       demonstrates usage as of cdrecord traditions
      telltoc_aquire_by_driveno()   demonstrates a scan-and-choose approach
-  With that aquired drive you can call
+  With that acquired drive you can call
      telltoc_media()   prints some information about the media in a drive
      telltoc_toc()     prints a table of content (if there is content)
      telltoc_msinfo()  prints parameters for mkisofs option -C
@@ -71,7 +72,7 @@ static unsigned int drive_count;
 static int drive_is_grabbed = 0;
 
 
-/* Some in-advance definitions to allow a more comprehensive ordering
+/* Some in-advance definitions to make possible a more comprehensive ordering
    of the functions and their explanations in here */
 int telltoc_aquire_by_adr(char *drive_adr);
 int telltoc_aquire_by_driveno(int *drive_no, int silent);
@@ -85,7 +86,7 @@ static int cd_is_audio = 0;           /* 0 = undecided , -1 = no , 1 = yes */
 
 /* ------------------------------- API gestures ---------------------------- */
 
-/** You need to aquire a drive before burning. The API offers this as one
+/** You need to acquire a drive before burning. The API offers this as one
     compact call and alternatively as application controllable gestures of
     whitelisting, scanning for drives and finally grabbing one of them.
 
@@ -127,7 +128,7 @@ int telltoc_aquire_by_adr(char *drive_adr)
 		return 0;
 	}
 
-	fprintf(stderr,"Aquiring drive '%s' ...\n", libburn_drive_adr);
+	fprintf(stderr,"Acquiring drive '%s' ...\n", libburn_drive_adr);
 	ret = burn_drive_scan_and_grab(&drive_list, libburn_drive_adr, 1);
 
 	if (ret <= 0) {
@@ -239,12 +240,14 @@ int telltoc_regrab(struct burn_drive *drive) {
 
 int telltoc_media(struct burn_drive *drive)
 {
-	int ret, media_found = 0, profile_no = -1;
+	int ret, media_found = 0, profile_no = -1, num_profiles = 0, i;
+	int profiles[64];
+	char is_current_profile[64];
 	double max_speed = 0.0, min_speed = 0.0, speed_conv;
 	off_t available = 0;
 	enum burn_disc_status s;
 	char profile_name[80], speed_unit[40];
-	struct burn_multi_caps *caps;
+	struct burn_multi_caps *caps = NULL;
 	struct burn_write_opts *o = NULL;
 
 	printf("Media current: ");
@@ -257,14 +260,31 @@ int telltoc_media(struct burn_drive *drive)
 	} else
 		printf("is not recognizable\n");
 
+	/* Determine speed unit before profile_name gets reused */
 	speed_conv = 176.4;
 	strcpy(speed_unit,"176.4 kB/s  (CD, data speed 150 KiB/s)");
 	if (strstr(profile_name, "DVD") == profile_name) {
 		speed_conv = 1385.0;
 		strcpy(speed_unit,"1385.0 kB/s  (DVD)");
-	}
+	} else if (strstr(profile_name, "BD") == profile_name) {
+                speed_conv = 4495.625;
+		strcpy(speed_unit,"4495.625 kB/s  (BD)");
+        }
 
-	/* >>> libburn does not obtain full profile list yet */
+	ret = burn_drive_get_all_profiles(drive, &num_profiles, profiles,
+					is_current_profile);
+	if (ret > 0) {
+		for (i = 0; i < num_profiles; i++) {
+			ret = burn_obtain_profile_name(profiles[i],
+							profile_name);
+			if (ret <= 0)
+				sprintf(profile_name,
+					"Unknown media type 0x%4.4X",
+					(unsigned int) profiles[i]);
+			printf("Drive can do : %s%s\n", profile_name,
+				is_current_profile[i] ? " (current)" : "");
+		}
+	}
 
 	printf("Media status : ");
 	s = burn_disc_get_status(drive);
@@ -296,15 +316,15 @@ int telltoc_media(struct burn_drive *drive)
 		/* Media appears writeable */
 		printf("Write multi  : ");
 		printf("%s multi-session , ",
-			 caps->multi_session == 1 ? "allows" : "prohibits");
+			 caps->multi_session == 1 ? "offers" : "cannot do");
 		if (caps->multi_track)
-			printf("allows multiple tracks\n");
+			printf("offers multiple tracks\n");
 		else
-			printf("enforces single track\n");
+			printf("offers only single track\n");
 		printf("Write start  : ");
 		if (caps->start_adr == 1)
 			printf(
-			"allows addresses [%.f , %.f]s , alignment=%.fs\n",
+			"offers addresses [%.f , %.f]s , alignment=%.fs\n",
 				(double) caps->start_range_low / 2048 ,
 				(double) caps->start_range_high / 2048 ,
 				(double) caps->start_alignment / 2048 );
@@ -379,6 +399,8 @@ int telltoc_media(struct burn_drive *drive)
 	}
 	printf("Speed unit 1x: %s\n", speed_unit);
 
+	if (caps != NULL)
+		burn_disc_free_multi_caps(&caps);
 	return 1;
 }
 
@@ -513,8 +535,8 @@ int telltoc_toc(struct burn_drive *drive)
 				cd_is_audio = -1;
 			}
 
-			printf("Media content: session %2d  ", session_no+1);
-			printf("track    %2d %s  lba: %9d  %4.2d:%2.2d:%2.2d\n",
+			printf("Media content: session %3d  ", session_no+1);
+			printf("track    %3d %s  lba: %9d  %4.2d:%2.2d:%2.2d\n",
 				track_count,
 				(track_is_audio ? "audio" : "data "),
 				lba, pmin, psec, pframe);
@@ -531,8 +553,8 @@ int telltoc_toc(struct burn_drive *drive)
 			pframe = toc_entry.pframe;
 			lba= burn_msf_to_lba(pmin, psec, pframe);
 		}
-		printf("Media content: session %2d  ", session_no+1);
-		printf("leadout            lba: %9d  %4.2d:%2.2d:%2.2d\n",
+		printf("Media content: session %3d  ", session_no+1);
+		printf("leadout             lba: %9d  %4.2d:%2.2d:%2.2d\n",
 			lba, pmin, psec, pframe);
 		last_track_size = lba - last_track_start;
 		telltoc_detect_cd(drive);
@@ -692,8 +714,16 @@ int telltoc_read_and_print(struct burn_drive *drive,
 print_result:;
 		total_count += data_count;
 		if (encoding == 1) {
-			if (data_count > 0)
-				fwrite(buf, data_count, 1, raw_fp);
+			if (data_count > 0) {
+				ret = fwrite(buf, data_count, 1, raw_fp);
+				if (ret < 1) {
+                   	 		fprintf(stderr,
+					    "FAILURE: writing to '%s' : %s\n",
+					    raw_file, strerror(errno));
+					fclose(raw_fp);
+					return 1;
+				}
+			}
 		} else for (i = 0; i < data_count; i += 16) {
 			if (encoding == 0) {
 				sprintf(line, "%8ds + %4d : ",
@@ -752,6 +782,8 @@ print_result:;
 		start_sector, sector_count,
 	        (int) (total_count / (off_t) sector_size));
 
+	if (raw_fp != NULL)
+		fclose(raw_fp);
 	return ret;
 }
 
@@ -776,6 +808,13 @@ int telltoc_setup(int argc, char **argv)
 {
     int i;
 
+    for (i = 1; i < argc; ++i) {
+        if (strlen(argv[i]) >= 4096) {
+            fprintf(stderr,
+                  "Argument at position %d is much too long. (Max 4095)\n", i);
+            return 2;
+        }
+    }
     for (i = 1; i < argc; ++i) {
         if (!strcmp(argv[i], "--drive")) {
             ++i;
@@ -817,7 +856,8 @@ int telltoc_setup(int argc, char **argv)
             sscanf(argv[i-2], "%d", &read_start);
             sscanf(argv[i-1], "%d", &read_count);
             print_encoding = 0;
-            if(strncmp(argv[i], "raw:", 4) == 0 || strcmp(argv[i],"1:") == 0) {
+            if(strncmp(argv[i], "raw:", 4) == 0 ||
+               strncmp(argv[i], "1:", 2) == 0) {
                 print_encoding = 1;
                 strcpy(print_raw_file, strchr(argv[i], ':') + 1);
 		if (strcmp(print_raw_file, "-") == 0) {
@@ -964,6 +1004,6 @@ finish_libburn:;
 }
 
 /*  License and copyright aspects:
-    See libburner.c
+    See test/libburner.c
 */
 
